@@ -4,7 +4,7 @@ package DBIx::SearchBuilder;
 use strict;
 use warnings;
 
-our $VERSION = "1.54";
+our $VERSION = "1.55";
 
 use Clone qw();
 use Encode qw();
@@ -128,8 +128,6 @@ sub CleanSlate {
     my $self = shift;
     $self->RedoSearch();
     $self->{'itemscount'}       = 0;
-    $self->{'tables'}           = "";
-    $self->{'auxillary_tables'} = "";
     $self->{'where_clause'}     = "";
     $self->{'limit_clause'}     = "";
     $self->{'order'}            = "";
@@ -1235,19 +1233,25 @@ that left joins work, this method needs a TABLE for the second field
 rather than merely an alias.  For this type of join, it will return
 the alias generated by the join.
 
-Instead of ALIAS1/FIELD1, it's possible to specify EXPRESSION, to join ALIAS2/TABLE2 on an arbitrary expression.
+Instead of ALIAS1/FIELD1, it's possible to specify EXPRESSION, to join
+ALIAS2/TABLE2 on an arbitrary expression.
+
+It is also possible to join to a pre-existing, already-limited
+L<DBIx::SearchBuilder> object, by passing it as COLLECTION2, instead
+of providing an ALIAS2 or TABLE2.
 
 =cut
 
 sub Join {
     my $self = shift;
     my %args = (
-        TYPE   => 'normal',
-        FIELD1 => undef,
-        ALIAS1 => 'main',
-        TABLE2 => undef,
-        FIELD2 => undef,
-        ALIAS2 => undef,
+        TYPE        => 'normal',
+        FIELD1      => undef,
+        ALIAS1      => 'main',
+        TABLE2      => undef,
+        COLLECTION2 => undef,
+        FIELD2      => undef,
+        ALIAS2      => undef,
         @_
     );
 
@@ -1553,21 +1557,7 @@ TODO: Why are they lowercased?
 =cut
 
 sub Fields {
-    my $self  = shift;
-    my $table = shift;
-
-    my $dbh = $self->_Handle->dbh;
-
-    # TODO: memoize this
-
-    return map lc( $_->[0] ), @{
-        eval {
-            $dbh->column_info( '', '', $table, '' )->fetchall_arrayref( [3] );
-          }
-          || $dbh->selectall_arrayref("DESCRIBE $table;")
-          || $dbh->selectall_arrayref("DESCRIBE \u$table;")
-          || []
-      };
+    return (shift)->_Handle->Fields( @_ );
 }
 
 
